@@ -9,11 +9,16 @@ try:
     patches = set()
     with open('debian/patches/series', 'r') as f:
         for line in f:
-            line = line.split('#', 1)[0]
-            if not line:
+            if line.startswith('#'):
                 patches.add(line.split('#')[1].strip())
                 continue
-            patches.add(line.split()[0])
+            line = line.split('#', 1)[0]
+            if not line:
+                continue
+            try:
+                patches.add(line.split()[0])
+            except IndexError:
+                continue
 except FileNotFoundError:
     sys.exit(0)
 
@@ -22,7 +27,7 @@ removed = set()
 for patch in os.listdir('debian/patches'):
     path = os.path.join('debian', 'patches', patch)
     # Don't delete the series file
-    if patch == 'series':
+    if patch in ('series', '00list'):
         continue
     # Ignore everything that is not a regular file
     if not os.path.isfile(path):
@@ -30,6 +35,9 @@ for patch in os.listdir('debian/patches'):
     # Ignore any README files
     if patch.startswith('README'):
         continue
+    # Ignore any notes files
+    if patch.startswith('notes'):
+        continue
     # Ignore everything that is listed in series
     if patch in patches:
         continue
