@@ -1,18 +1,24 @@
 #!/usr/bin/python3
 
+import sys
+
 from lintian_brush.lintian_overrides import (
     update_overrides,
     Override,
     )
 
-
-renames = {
-    'debian-watch-may-check-gpg-signature':
-        'debian-watch-does-not-check-gpg-signature',
-    'systemd-no-service-for-init-script':
-        'omitted-systemd-service-for-init.d-script',
-    'rules-requires-root-implicitly': 'rules-requires-root-missing',
-}
+renames = {}
+try:
+    with open('/usr/share/lintian/data/override/renamed-tags', 'r') as f:
+        for line in f:
+            if line.startswith('#'):
+                continue
+            if not line.strip():
+                continue
+            (old, new) = line.split('=>')
+            renames[old.strip()] = new.strip()
+except FileNotFoundError:
+    sys.exit(2)
 
 
 def rename_override_tags(override):
