@@ -22,6 +22,8 @@ __all__ = ['guess_repository_url', 'determine_browser_url']
 import re
 from urllib.parse import urlparse
 
+from .vcs import split_vcs_url
+
 
 MAINTAINER_EMAIL_MAP = {
     'pkg-javascript-devel@lists.alioth.debian.org': 'js-team',
@@ -117,6 +119,16 @@ TEAM_NAME_MAP = {
     'pkg-apache': 'apache-team',
 }
 
+GIT_PATH_RENAMES = {
+    'pkg-kde/applications': 'qt-kde-team/kde',
+    '3dprinter/packages': '3dprinting-team',
+    'pkg-emacsen/pkg': 'emacsen-team',
+    'debian-astro/packages': 'debian-astro-team',
+    'debian-islamic/packages': 'islamic-team',
+    'debichem/packages': 'debichem-team',
+    'pkg-privacy/packages': 'pkg-privacy-team',
+}
+
 
 def guess_repository_url(package, maintainer_email):
     """Guess the repository URL for a package hosted on Salsa.
@@ -146,18 +158,19 @@ def determine_browser_url(url):
     Returns:
       a browser URL
     """
-    if ' -b ' in url:
-        (url, branch) = url.split(' -b ', 1)
-    else:
-        branch = None
+    (url, branch, subpath) = split_vcs_url(url)
     parsed_url = urlparse(url)
     # TODO(jelmer): Add support for branches
     assert parsed_url.netloc == 'salsa.debian.org'
     path = parsed_url.path
     if path.endswith('.git'):
         path = path[:-len('.git')]
+    if subpath and not branch:
+        branch = "HEAD"
     if branch:
         path = path + '/tree/%s' % branch
+    if subpath:
+        path = path + '/' + subpath
     return 'https://salsa.debian.org%s' % path
 
 
@@ -182,6 +195,20 @@ def salsa_url_from_alioth_url(vcs_type, alioth_url):
         m = "(https?|git)://(anonscm|git).debian.org/(cgit/|git/)?users/"
         if re.match(m, alioth_url):
             return re.sub(m, 'https://salsa.debian.org/', alioth_url)
+        m = re.match(
+            "(https?|git)://(anonscm|git).debian.org/(cgit/|git/)?(.+)",
+            alioth_url)
+        if m:
+            parts = m.group(4).split('/')
+            for i in range(len(parts), 0, -1):
+                subpath = '/'.join(parts[:i])
+                try:
+                    return (
+                        'https://salsa.debian.org/' +
+                        GIT_PATH_RENAMES[subpath] + '/' +
+                        '/'.join(parts[i:]))
+                except KeyError:
+                    pass
         m = re.match(
             "(https?|git)://(anonscm|git).debian.org/(cgit/|git/)?([^/]+)/",
             alioth_url)
