@@ -65,6 +65,16 @@ class DetermineBrowserUrlTests(TestCase):
             determine_browser_url(
                 'https://salsa.debian.org/js-team/node-blah -b debian'))
 
+    def test_subpath(self):
+        self.assertEqual(
+            'https://salsa.debian.org/js-team/node-blah/tree/debian/foo',
+            determine_browser_url(
+                'https://salsa.debian.org/js-team/node-blah -b debian [foo]'))
+        self.assertEqual(
+            'https://salsa.debian.org/js-team/node-blah/tree/HEAD/foo',
+            determine_browser_url(
+                'https://salsa.debian.org/js-team/node-blah [foo]'))
+
 
 class SalsaUrlFromAliothUrlTests(TestCase):
 
@@ -144,6 +154,39 @@ class SalsaUrlFromAliothUrlTests(TestCase):
             salsa_url_from_alioth_url(
                 'git',
                 'git://anonscm.debian.org/users/brlink/xwit.git'))
+        self.assertEqual(
+            'https://salsa.debian.org/qt-kde-team/kde/kruler',
+            salsa_url_from_alioth_url(
+                'git',
+                'https://anonscm.debian.org/git/pkg-kde/applications/'
+                'kruler'))
+        self.assertEqual(
+            'https://salsa.debian.org/3dprinting-team/arduino-mighty',
+            salsa_url_from_alioth_url(
+                'git',
+                'https://anonscm.debian.org/git/3dprinter/packages/'
+                'arduino-mighty'))
+        self.assertEqual(
+            'https://salsa.debian.org/emacsen-team/lua-mode',
+            salsa_url_from_alioth_url(
+                'git',
+                'https://anonscm.debian.org/git/pkg-emacsen/pkg/lua-mode'))
+        self.assertEqual(
+            'https://salsa.debian.org/debian-astro-team/astromatic.git',
+            salsa_url_from_alioth_url(
+                'git',
+                'http://anonscm.debian.org/cgit/debian-astro/packages/'
+                'astromatic.git'))
+        self.assertEqual(
+            'https://salsa.debian.org/debichem-team/bkchem.git',
+            salsa_url_from_alioth_url(
+                'git',
+                'https://anonscm.debian.org/git/debichem/packages/bkchem.git'))
+        self.assertEqual(
+            'https://salsa.debian.org/3dprinting-team/yagv.git',
+            salsa_url_from_alioth_url(
+                'git',
+                'https://anonscm.debian.org/cgit/3dprinter/packages/yagv.git'))
 
     def test_svn(self):
         self.assertEqual(
