@@ -22,7 +22,6 @@ from unittest import TestCase
 from lintian_brush.vcs import (
     canonicalize_vcs_url,
     determine_browser_url,
-    extract_vcs_url_branch,
     fixup_broken_git_url,
     plausible_url,
     sanitize_url,
@@ -92,20 +91,30 @@ class FixUpGitUrlTests(TestCase):
             'https://github.com/blah/blah -b master',
             fixup_broken_git_url('https://github.com/blah/blah/tree/master'))
 
+    def test_freedesktop(self):
+        self.assertEqual(
+            'https://gitlab.freedesktop.org/xorg/xserver',
+            fixup_broken_git_url('git://anongit.freedesktop.org/xorg/xserver'))
+
+    def test_gnome(self):
+        self.assertEqual(
+            'https://gitlab.gnome.org/GNOME/alacarte',
+            fixup_broken_git_url('https://git.gnome.org/browse/alacarte'))
+
 
 class SanitizeUrlTests(TestCase):
 
     def test_simple(self):
         self.assertEqual(
-            'http://github.com/jelmer/blah',
+            'https://github.com/jelmer/blah.git',
             sanitize_url('http://github.com/jelmer/blah'))
 
     def test_git_http(self):
         self.assertEqual(
-            'http://github.com/jelmer/blah',
+            'https://github.com/jelmer/blah.git',
             sanitize_url('git+http://github.com/jelmer/blah'))
         self.assertEqual(
-            'https://github.com/jelmer/blah',
+            'https://github.com/jelmer/blah.git',
             sanitize_url('git+https://github.com/jelmer/blah'))
 
 
@@ -150,30 +159,6 @@ class PlausibleUrlTests(TestCase):
         self.assertTrue(plausible_url('https://foo/blah'))
 
 
-class ExtractVcsUrlBranchTests(TestCase):
-
-    def test_none(self):
-        self.assertEqual(
-            ('https://github.com/jelmer/example', None),
-            extract_vcs_url_branch('https://github.com/jelmer/example'))
-        self.assertEqual(
-            ('https://github.com/jelmer/example [path/to/packaging]', None),
-            extract_vcs_url_branch(
-                'https://github.com/jelmer/example [path/to/packaging]'))
-
-    def test_branch(self):
-        self.assertEqual(
-            ('https://github.com/jelmer/example [path/to/packaging]',
-                'master'),
-            extract_vcs_url_branch(
-                'https://github.com/jelmer/example [path/to/packaging] '
-                '-b master'))
-        self.assertEqual(
-            ('https://github.com/jelmer/example', 'master'),
-            extract_vcs_url_branch(
-                'https://github.com/jelmer/example -b master'))
-
-
 class SplitVcsUrlTests(TestCase):
 
     def test_none(self):
