@@ -23,7 +23,7 @@ import re
 import subprocess
 import tempfile
 import urllib.error
-from urllib.parse import urlparse, urlunparse, urljoin
+from urllib.parse import quote, urlparse, urlunparse, urljoin
 from warnings import warn
 
 from debian.deb822 import Deb822
@@ -190,10 +190,12 @@ def guess_repo_from_url(url, net_access=False):
 
 
 def known_bad_guess(datum):
-    if datum.field == 'Bug-Submit':
+    if datum.field in ('Bug-Submit', 'Bug-Database'):
         parsed_url = urlparse(datum.value)
         if parsed_url.hostname == 'bugzilla.gnome.org':
             return True
+        if parsed_url.hostname == 'bugs.freedesktop.org':
+            return True
     return False
 
 
@@ -202,12 +204,15 @@ def filter_bad_guesses(guessed_items):
 
 
 def update_from_guesses(upstream_metadata, guessed_items):
+    changed = False
     for datum in guessed_items:
         current_datum = upstream_metadata.get(datum.field)
         if not current_datum or (
                 certainty_to_confidence(datum.certainty) <
                 certainty_to_confidence(current_datum.certainty)):
             upstream_metadata[datum.field] = datum
+            changed = True
+    return changed
 
 
 def read_python_pkg_info(path):
@@ -574,6 +579,10 @@ def guess_from_doap(path, trust_package):
             url = extract_url(child)
             if url:
                 yield UpstreamDatum('Homepage', url, 'certain')
+        if child.tag == ('{%s}download-page' % DOAP_NAMESPACE):
+            url = extract_url(child)
+            if url:
+                yield UpstreamDatum('X-Download', url, 'certain')
         if child.tag == ('{%s}repository' % DOAP_NAMESPACE):
             for repo in child:
                 if repo.tag in (
@@ -885,6 +894,14 @@ def guess_bug_database_url_from_repo_url(url):
         return urlunparse(
             ('https', 'github.com', path,
              None, None, None))
+    if is_gitlab_site(parsed_url.hostname):
+        path = '/'.join(parsed_url.path.split('/')[:3])
+        if path.endswith('.git'):
+            path = path[:-4]
+        path = path + '/issues'
+        return urlunparse(
+            ('https', parsed_url.hostname, path,
+             None, None, None))
     return None
 
 
@@ -923,13 +940,26 @@ def verify_bug_database_url(url):
             path_elements[0], path_elements[1])
         data = _load_json_url(api_url)
         return data['has_issues']
+    if is_gitlab_site(parsed_url.netloc):
+        path_elements = parsed_url.path.strip('/').split('/')
+        if len(path_elements) < 3 or path_elements[-1] != 'issues':
+            return False
+        api_url = 'https://%s/api/v4/projects/%s/issues' % (
+            parsed_url.netloc, quote('/'.join(path_elements[:-1]), safe=''))
+        try:
+            data = _load_json_url(api_url)
+        except urllib.error.HTTPError as e:
+            if e.status == 404:
+                return
+            raise
+        return len(data) > 0
     return None
 
 
 def verify_bug_submit_url(url):
     parsed_url = urlparse(url)
-    if parsed_url.netloc == 'gitlab.com':
-        path = '/'.join(parsed_url.path.strip('/').split('/')[:2])
+    if parsed_url.netloc == 'gitlab.com' or is_gitlab_site(parsed_url.netloc):
+        path = '/'.join(parsed_url.path.strip('/').split('/')[:-1])
         return verify_bug_database_url(
             urlunparse(parsed_url._replace(path=path)))
     return None
@@ -942,6 +972,13 @@ def _extrapolate_repository_from_homepage(upstream_metadata, net_access):
         return UpstreamDatum('Repository', repo, 'likely')
 
 
+def _extrapolate_repository_from_download(upstream_metadata, net_access):
+    repo = guess_repo_from_url(
+            upstream_metadata['X-Download'].value, net_access=net_access)
+    if repo:
+        return UpstreamDatum('Repository', repo, 'likely')
+
+
 def _extrapolate_repository_from_bug_db(upstream_metadata, net_access):
     repo = guess_repo_from_url(
         upstream_metadata['Bug-Database'].value, net_access=net_access)
@@ -1009,6 +1046,7 @@ EXTRAPOLATE_FNS = [
     ('Repository', 'Bug-Database',
      _extrapolate_bug_database_from_repository),
     ('Bug-Database', 'Bug-Submit', _extrapolate_bug_submit_from_bug_db),
+    ('X-Download', 'Repository', _extrapolate_repository_from_download),
 ]
 
 
@@ -1048,12 +1086,23 @@ def extend_upstream_metadata(upstream_metadata, path, minimum_certainty=None,
     while changed:
         changed = False
         for from_field, to_field, fn in EXTRAPOLATE_FNS:
-            if (to_field in upstream_metadata or
-                    from_field not in upstream_metadata):
+            try:
+                from_value = upstream_metadata[from_field]
+            except KeyError:
+                continue
+            old_to_value = upstream_metadata.get(to_field)
+            if old_to_value is not None and (
+                    certainty_to_confidence(from_value.certainty) >=
+                    certainty_to_confidence(old_to_value.certainty)):
                 continue
             result = fn(upstream_metadata, net_access)
-            if result and certainty_sufficient(
-                    result.certainty, minimum_certainty):
+            if not result:
+                continue
+            if not certainty_sufficient(result.certainty, minimum_certainty):
+                continue
+            if old_to_value is None or (
+                    certainty_to_confidence(result.certainty) <
+                    certainty_to_confidence(old_to_value.certainty)):
                 upstream_metadata[to_field] = result
                 changed = True
 
@@ -1071,6 +1120,7 @@ def probe_upstream_branch_url(url, version=None):
         b = Branch.open(url)
         b.last_revision()
         if version is not None:
+            version = version.split('+git')[0]
             tag_names = b.tags.get_tag_dict().keys()
             if not tag_names:
                 # Uhm, hmm
@@ -1079,11 +1129,15 @@ def probe_upstream_branch_url(url, version=None):
                 return True
             if 'v%s' % version in tag_names:
                 return True
+            if version.replace('.', '_') in tag_names:
+                return True
             for tag_name in tag_names:
                 if tag_name.endswith('_' + version):
                     return True
                 if tag_name.endswith('-' + version):
                     return True
+                if tag_name.endswith('_%s' % version.replace('.', '_')):
+                    return True
             return False
         else:
             return True
