@@ -20,7 +20,6 @@
 __all__ = [
     'fixup_broken_git_url',
     'sanitize_url',
-    'extract_vcs_url_branch',
     'split_vcs_url',
     'determine_browser_url',
     ]
@@ -39,12 +38,9 @@ KNOWN_GITLAB_SITES = [
     ]
 
 
-def extract_vcs_url_branch(url):
-    # Deprecated, use split_vcs_url
-    (repo_url, branch, subpath) = split_vcs_url(url)
-    if subpath:
-        repo_url = '%s [%s]' % (repo_url, subpath)
-    return repo_url, branch
+def is_gitlab_site(hostname, net_access=False):
+    # TODO(jelmer): Try API to see if this is a gitlab site
+    return hostname in KNOWN_GITLAB_SITES
 
 
 def split_vcs_url(url):
@@ -75,8 +71,15 @@ def unsplit_vcs_url(repo_url, branch=None, subpath=None):
 def sanitize_url(url):
     url = url.strip()
     if url.startswith('git+http:') or url.startswith('git+https:'):
-        return url[4:]
-    return fixup_broken_git_url(url)
+        url = url[4:]
+    if url.startswith('hg+https:') or url.startswith('hg+http'):
+        url = url[3:]
+    if url.startswith('bzr+lp:') or url.startswith('bzr+http'):
+        url = url.split('+', 1)[1]
+    url = fixup_broken_git_url(url)
+    url = canonical_vcs_git_url(url)
+    url = find_secure_vcs_url(url, net_access=False) or url
+    return url
 
 
 def plausible_url(url):
@@ -87,7 +90,7 @@ def fix_path_in_port(parsed, branch):
     if ':' not in parsed.netloc or parsed.netloc.endswith(']'):
         return None, None
     host, port = parsed.netloc.rsplit(':', 1)
-    if host.split('@')[-1] not in ('salsa.debian.org', 'github.com'):
+    if host.split('@')[-1] not in (KNOWN_GITLAB_SITES + ['github.com']):
         return None, None
     if not port or port.isdigit():
         return None, None
@@ -96,8 +99,8 @@ def fix_path_in_port(parsed, branch):
         netloc=host), branch
 
 
-def fix_salsa_scheme(parsed, branch):
-    if parsed.hostname == 'salsa.debian.org':
+def fix_gitlab_scheme(parsed, branch):
+    if is_gitlab_site(parsed.hostname):
         return parsed._replace(scheme='https'), branch
     return None, None
 
@@ -109,8 +112,8 @@ def fix_salsa_cgit_url(parsed, branch):
     return None, None
 
 
-def fix_salsa_tree_in_url(parsed, branch):
-    if parsed.hostname == 'salsa.debian.org':
+def fix_gitlab_tree_in_url(parsed, branch):
+    if is_gitlab_site(parsed.hostname):
         parts = parsed.path.split('/')
         if len(parts) >= 5 and parts[3] == 'tree':
             branch = '/'.join(parts[4:])
@@ -151,9 +154,21 @@ def fix_branch_argument(parsed, branch):
 
 def fix_git_gnome_org_url(parsed, branch):
     if parsed.netloc == 'git.gnome.org':
+        if parsed.path.startswith('/browse'):
+            path = parsed.path[7:]
+        else:
+            path = parsed.path
         parsed = parsed._replace(
             netloc='gitlab.gnome.org', scheme='https',
-            path='/GNOME' + parsed.path)
+            path='/GNOME' + path)
+        return parsed, branch
+    return None, None
+
+
+def fix_freedesktop_org_url(parsed, branch):
+    if parsed.netloc == 'anongit.freedesktop.org':
+        parsed = parsed._replace(
+            netloc='gitlab.freedesktop.org', scheme='https')
         return parsed, branch
     return None, None
 
@@ -163,13 +178,14 @@ def fixup_broken_git_url(url):
 
     A common misspelling is to add an extra ":" after the hostname
     """
-    repo_url, branch = extract_vcs_url_branch(url)
+    repo_url, branch, subpath = split_vcs_url(url)
 
     parsed = urlparse(repo_url)
     changed = False
-    for fn in [fix_path_in_port, fix_salsa_scheme, fix_salsa_cgit_url,
-               fix_salsa_tree_in_url, fix_double_slash, fix_extra_colon,
-               drop_git_username, fix_branch_argument, fix_git_gnome_org_url]:
+    for fn in [fix_path_in_port, fix_gitlab_scheme, fix_salsa_cgit_url,
+               fix_gitlab_tree_in_url, fix_double_slash, fix_extra_colon,
+               drop_git_username, fix_branch_argument, fix_git_gnome_org_url,
+               fix_freedesktop_org_url]:
         newparsed, newbranch = fn(parsed, branch)
         if newparsed:
             changed = True
@@ -177,7 +193,7 @@ def fixup_broken_git_url(url):
             branch = newbranch
 
     if changed:
-        return unsplit_vcs_url(urlunparse(parsed), branch)
+        return unsplit_vcs_url(urlunparse(parsed), branch, subpath)
     return url
 
 
@@ -194,7 +210,7 @@ def browse_url_from_repo_url(url):
         return urlunparse(
             ('https', 'code.launchpad.net', parsed_url.path,
              parsed_url.query, parsed_url.params, parsed_url.fragment))
-    if parsed_url.netloc in KNOWN_GITLAB_SITES:
+    if is_gitlab_site(parsed_url.netloc):
         path = parsed_url.path
         if path.endswith('.git'):
             path = path[:-4]
@@ -266,7 +282,8 @@ def canonicalize_vcs_browser_url(url):
 def canonical_vcs_git_url(url):
     (repo_url, branch, subpath) = split_vcs_url(url)
     parsed_url = urlparse(repo_url)
-    if parsed_url.netloc in ('salsa.debian.org', 'github.com'):
+    if (is_gitlab_site(parsed_url.netloc) or
+            parsed_url.netloc in ['github.com']):
         if not parsed_url.path.rstrip('/').endswith('.git'):
             parsed_url = parsed_url._replace(
                 path=parsed_url.path.rstrip('/') + '.git')
@@ -285,3 +302,57 @@ def canonicalize_vcs_url(vcs_type, url):
         return canonicalize_vcs_fns[vcs_type](url)
     except KeyError:
         return url
+
+
+def try_open_branch(url, branch_name=None):
+    import breezy.ui
+    from breezy.controldir import ControlDir
+    old_ui = breezy.ui.ui_factory
+    breezy.ui.ui_factory = breezy.ui.SilentUIFactory()
+    try:
+        c = ControlDir.open(url)
+        b = c.open_branch(name=branch_name)
+        b.last_revision()
+        return b
+    except Exception:
+        # TODO(jelmer): Catch more specific exceptions?
+        return None
+    finally:
+        breezy.ui.ui_factory = old_ui
+
+
+SECURE_SCHEMES = ['https', 'git+ssh', 'bzr+ssh', 'hg+ssh', 'ssh', 'svn+ssh']
+
+
+def find_secure_vcs_url(url, net_access=True):
+    (repo_url, branch, subpath) = split_vcs_url(url)
+    parsed_repo_url = urlparse(repo_url)
+    if parsed_repo_url.scheme in SECURE_SCHEMES:
+        return url
+
+    # Sites we know to be available over https
+    if (is_gitlab_site(parsed_repo_url.hostname, net_access) or
+            parsed_repo_url.hostname in [
+                'github.com', 'git.launchpad.net', 'bazaar.launchpad.net',
+                'code.launchpad.net']):
+        parsed_repo_url = parsed_repo_url._replace(scheme='https')
+
+    if parsed_repo_url.scheme == 'lp':
+        parsed_repo_url = parsed_repo_url._replace(
+            scheme='https', netloc='code.launchpad.net')
+
+    if net_access:
+        secure_repo_url = parsed_repo_url._replace(scheme='https')
+        insecure_branch = try_open_branch(repo_url, branch)
+        secure_branch = try_open_branch(urlunparse(parsed_repo_url), branch)
+        if secure_branch:
+            if (not insecure_branch or
+                    secure_branch.last_revision() ==
+                    insecure_branch.last_revision()):
+                parsed_repo_url = secure_repo_url
+
+    if parsed_repo_url.scheme in SECURE_SCHEMES:
+        return unsplit_vcs_url(urlunparse(parsed_repo_url), branch, subpath)
+
+    # Can't find a secure URI :(
+    return None
